package org.gcube.portlets.user.geoportaldataviewer.client.events;

import java.util.List;

import org.gcube.application.geoportalcommon.shared.geoportal.DocumentDV;

import com.google.gwt.event.shared.GwtEvent;

/**
 * The Class SearchPerformedEvent.
 *
 * @author Francesco Mangiacrapa at ISTI-CNR francesco.mangiacrapa@isti.cnr.it
 * 
 *         Dec 14, 2021
 */
public class SearchPerformedEvent extends GwtEvent<SearchPerformedEventHandler> {
	public static Type<SearchPerformedEventHandler> TYPE = new Type<SearchPerformedEventHandler>();
	private List<? extends DocumentDV> data;
	private boolean searchReset;

	/**
	 * Instantiates a new search performed event.
	 *
	 * @param data        the data
	 * @param searchReset the search reset
	 */
	public SearchPerformedEvent(List<? extends DocumentDV> data, boolean searchReset) {
		this.data = data;
		this.searchReset = searchReset;
	}

	/**
	 * Gets the associated type.
	 *
	 * @return the associated type
	 */
	@Override
	public Type<SearchPerformedEventHandler> getAssociatedType() {
		return TYPE;
	}

	/**
	 * Dispatch.
	 *
	 * @param handler the handler
	 */
	@Override
	protected void dispatch(SearchPerformedEventHandler handler) {
		handler.onSearchDone(this);

	}

	/**
	 * Gets the data.
	 *
	 * @return the data
	 */
	public List<? extends DocumentDV> getData() {
		return data;
	}

	/**
	 * Checks if is search reset.
	 *
	 * @return true, if is search reset
	 */
	public boolean isSearchReset() {
		return searchReset;
	}

}