package org.gcube.portlets.user.geoportaldataviewer.client.events;

import org.gcube.application.geoportalcommon.shared.geoportal.DocumentDV;

import com.google.gwt.event.shared.GwtEvent;

/**
 * The Class ShowDetailsEvent.
 *
 * @author Francesco Mangiacrapa at ISTI-CNR francesco.mangiacrapa@isti.cnr.it
 * 
 *         Jul 30, 2021
 */
public class ShowPopupOnCentroiEvent extends GwtEvent<ShowPopupOnCentroiEventHandler> {
	public static Type<ShowPopupOnCentroiEventHandler> TYPE = new Type<ShowPopupOnCentroiEventHandler>();
	private DocumentDV document;
	private Double centroidLong;
	private Double centroidLat;

	/**
	 * Instantiates a new show details event.
	 *
	 * @param geonaItemType the geona item type
	 * @param geonaMongoID  the geona mongo ID
	 * @param itemName      the item name
	 * @param featureRow    the feature row
	 */
	public ShowPopupOnCentroiEvent(DocumentDV document, Double centroidLong, Double centroidLat) {
		this.document = document;
		this.centroidLong = centroidLong;
		this.centroidLat = centroidLat;

	}

	/**
	 * Gets the associated type.
	 *
	 * @return the associated type
	 */
	@Override
	public Type<ShowPopupOnCentroiEventHandler> getAssociatedType() {
		return TYPE;
	}

	/**
	 * Dispatch.
	 *
	 * @param handler the handler
	 */
	@Override
	protected void dispatch(ShowPopupOnCentroiEventHandler handler) {
		handler.onShowPopup(this);

	}

	public DocumentDV getDocument() {
		return document;
	}
	
	public Double getCentroidLat() {
		return centroidLat;
	}
	
	public Double getCentroidLong() {
		return centroidLong;
	}

}